from pbr.version import VersionInfo

# A hand agent to use for testing
__version__ = VersionInfo('sparql_slurper')
UserAgent = f"{__version__.package}/{__version__.version_string()} " \
            f"(https://github.com/hsolbrig/PyShEx; solbrig@jhu.edu)"